UNIT readinput;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   Reading input file with input model parameters for the multiple Einstein method.
   This parser eliminates some mistakes with user-input. Not perfect but helps a lot
 *)

INTERFACE
USES CRT,sysutils,variables;

VAR LINE:ANSISTRING;                    (* A line of the input file *)
    LENGTHLINE:INTEGER;                 (* Length of a line in the input file *)


FUNCTION CALC_LENGTH(S:ANSISTRING):INTEGER;
PROCEDURE GET_STOP_CALC_KEY;
PROCEDURE SET_READ_FILE(VAR FIL:TEXT;FILENAME:ANSISTRING);
PROCEDURE SET_WRITE_FILE(VAR FIL:TEXT;FILENAME:ANSISTRING);
FUNCTION EXIST_FILE(TESTNAME:ANSISTRING):BOOLEAN;
PROCEDURE SLIDE_LINE_ITEM(INP_S,ITEM:ANSISTRING;VAR FOUND:BOOLEAN;VAR J:INTEGER);
PROCEDURE DETECTBLANK(INP_S:ANSISTRING;VAR STRT:INTEGER);
PROCEDURE DETECTNOTBLANK(INP_S:ANSISTRING;VAR STRT:INTEGER);
PROCEDURE GET_PATH_INPUTFILE(VAR NAME_INPUTFILE:ANSISTRING);
PROCEDURE READ_INPUT_FILE(VAR INPUT_OKAY:BOOLEAN);
IMPLEMENTATION


FUNCTION CALC_LENGTH(S:ANSISTRING):INTEGER;
(* Determining the correct length of a string S read from a file.
   When the ordinal of s[length(s)] equals 13 we have length(s)-1
   characters on the line. If not we have length(s) characters
 *)
VAR L,M:INTEGER;V:CHAR;
BEGIN
  L:=LENGTH(S);
  IF L>0 THEN
  BEGIN
    V:=S[L]; M:=ORD(V);
    IF (M=10) OR (M=13) THEN CALC_LENGTH:=L-1 
    ELSE CALC_LENGTH:=L
  END
  ELSE CALC_LENGTH:=0
END;

PROCEDURE GET_STOP_CALC_KEY;
(* Switches stop_calc_key on or off. It is used in the calculation unit to see if
   a calculation must be stopped. Stop_calc_key is a global, true stops the calculation
   and false continues it. It is set True when escape key is hit
 *)
VAR CH:CHAR;
BEGIN
  IF KEYPRESSED THEN
  BEGIN
    CH:=READKEY;
    IF ORD(CH)=27 THEN STOP_CALC_KEY:=TRUE ELSE STOP_CALC_KEY:=FALSE
  END
END;

PROCEDURE SET_READ_FILE(VAR FIL:TEXT;FILENAME:ANSISTRING);
(* Assign a name of file to a file and set it open for reading *)
BEGIN
  ASSIGN(FIL,FILENAME);RESET(FIL)
END;

PROCEDURE SET_WRITE_FILE(VAR FIL:TEXT;FILENAME:ANSISTRING);
(* Assign a name of file to a file and set it open for writing *)
BEGIN
  ASSIGN(FIL,FILENAME);REWRITE(FIL)
END;

FUNCTION EXIST_FILE(TESTNAME:ANSISTRING):BOOLEAN;
(* Check if a file exists *)
VAR FF:TEXT;
BEGIN
  ASSIGN(FF,TESTNAME);
  {$I-}  RESET(FF);  {$I+}
  IF IORESULT=0 THEN
  BEGIN
    EXIST_FILE:=TRUE;CLOSE(FF)
  END
  ELSE EXIST_FILE:=FALSE
END;

PROCEDURE SLIDE_LINE_ITEM(INP_S,ITEM:ANSISTRING;VAR FOUND:BOOLEAN;VAR J:INTEGER);
(* Searches an ITEM through the whole string INP_S until it has been found
   starting at the jth character.
 *)
VAR L,LENG_INP_S:INTEGER;S:ANSISTRING;
BEGIN
  L:=LENGTH(ITEM);LENG_INP_S:=CALC_LENGTH(INP_S);
  S:='';FOUND:=FALSE;
  IF LENG_INP_S>=L THEN
  REPEAT
    IF LENGTH(S)>=L THEN DELETE(S,1,1);
    S:=S+INP_S[J];J:=J+1;
    IF S=ITEM THEN FOUND:=TRUE
  UNTIL
    (FOUND=TRUE) OR (J>LENG_INP_S)
END;

PROCEDURE DETECTBLANK(INP_S:ANSISTRING;VAR STRT:INTEGER);
(* Finds a blank in the string INP_S starting at character at position STRT.
 *)
VAR LENG_INP_S:INTEGER;STOP_IT:BOOLEAN;
BEGIN
  STRT:=STRT-1;LENG_INP_S:=CALC_LENGTH(INP_S); STOP_IT:=FALSE;
  REPEAT
    STRT:=STRT+1;
    IF STRT<=LENG_INP_S THEN
    BEGIN
      IF (INP_S[STRT]=' ') OR (ORD(INP_S[STRT])=0) THEN STOP_IT:=TRUE
    END
  UNTIL
    (STOP_IT=TRUE) OR (STRT>=LENG_INP_S)
END;

PROCEDURE DETECTNOTBLANK(INP_S:ANSISTRING;VAR STRT:INTEGER);
(* Finds a character not equal to blank in a string INP_S starting at
   character at position STRT of INP_S.
 *)
VAR LENG_INP_S:INTEGER;STOP_IT:BOOLEAN;
BEGIN
  STRT:=STRT-1;LENG_INP_S:=CALC_LENGTH(INP_S); STOP_IT:=FALSE;
  REPEAT
    STRT:=STRT+1;
    IF STRT<=LENG_INP_S THEN
    BEGIN
      IF (INP_S[STRT]<>' ') AND (ORD(INP_S[STRT])<>0) THEN STOP_IT:=TRUE
    END
  UNTIL
    (STOP_IT=TRUE) OR (STRT>=LENG_INP_S)
END;

PROCEDURE GET_PATH_INPUTFILE(VAR NAME_INPUTFILE:ANSISTRING);
(* Gets complete name of input file, NAME_INPUTFILE, including its path *)
VAR PATH:ANSISTRING;
    J,LENG:INTEGER;
    HAS_SLASH:BOOLEAN;
BEGIN
  (* Check if name of the input file has a path or not *)
  LENG:=LENGTH(NAME_INPUTFILE); HAS_SLASH:=FALSE;
  IF LENG>0 THEN
  BEGIN
    FOR J:=1 TO LENG DO
      IF NAME_INPUTFILE[J]=WHAT_SLASH THEN HAS_SLASH:=TRUE
  END;
  (* When forward slash present, HAS_SLASH=TRUE, do not change the name
     of the input file. When FALSE assume path is the same as current directory
   *)
  IF HAS_SLASH=FALSE THEN
  BEGIN
    (* No forward slash: input file has been called from current directory
     *)
    PATH:=GETCURRENTDIR;
    NAME_INPUTFILE:=PATH+WHAT_SLASH+NAME_INPUTFILE
  END
END;

PROCEDURE READLINE(VAR LINE:ANSISTRING;VAR NUM_LINE:INTEGER);
(* Returns the next line not preceding with a comment marker '#' *)
VAR J:INTEGER;
    STOP_IT:BOOLEAN;
BEGIN
  STOP_IT:=FALSE;
  REPEAT
    READLN(F_INPUT,LINE); NUM_LINE:=NUM_LINE+1; LENGTHLINE:=CALC_LENGTH(LINE);
    IF LENGTHLINE>0 THEN
    BEGIN
      J:=1; DETECTNOTBLANK(LINE,J);
      IF LINE[J]<>'#' THEN STOP_IT:=TRUE
    END
  UNTIL
    (EOF(F_INPUT)) OR (STOP_IT=TRUE)
END;

PROCEDURE GET_STRING(VAR STRIN:ANSISTRING);
(* Gets string at the beginning of a line, e.g name of the substance *)
VAR J:INTEGER;
BEGIN
  J:=1; DETECTNOTBLANK(LINE,J); STRIN:='';
  REPEAT
    IF (LINE[J]<>' ') AND (J<=LENGTHLINE) THEN STRIN:=STRIN+LINE[J];
    J:=J+1;
  UNTIL
    (LINE[J]=' ') OR (J>LENGTHLINE)
END;

PROCEDURE GET_PARAMETER_VALUE(VAR STRT:INTEGER;VAR X:EXTENDED;VAR INPUT_OKAY:BOOLEAN);
(* Gets value from a line starting at strt-st character *)
VAR STRIN:ANSISTRING;
    J,CODE:INTEGER;
BEGIN
  J:=STRT; DETECTNOTBLANK(LINE,J); STRIN:='';
  REPEAT
    IF (LINE[J]<>' ') AND (J<=LENGTHLINE) THEN STRIN:=STRIN+LINE[J];
    J:=J+1;
  UNTIL
    (LINE[J]=' ') OR (J>LENGTHLINE);
  VAL(STRIN,X,CODE);
  IF CODE<>0 THEN INPUT_OKAY:=FALSE;
  STRT:=J
END;

PROCEDURE GET_VIBRATIONAL(N_E:INTEGER;VAR INPUT_OKAY:BOOLEAN);
(* Get vibrational parameters *)
VAR X:EXTENDED;
    J:INTEGER;
BEGIN
  (* Number, N_E *)
  J:=1; GET_PARAMETER_VALUE(J,X,INPUT_OKAY);
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Einstein temperature *)
    GET_PARAMETER_VALUE(J,X,INPUT_OKAY); THETA0[N_E]:=X
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Fraction *)
    GET_PARAMETER_VALUE(J,X,INPUT_OKAY); FRAC[N_E]:=X
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Gruneisen parameter gamma_0 *)
    GET_PARAMETER_VALUE(J,X,INPUT_OKAY); GAMMA0[N_E]:=X
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Mode-q(m) parameter q0, or m *)
    GET_PARAMETER_VALUE(J,X,INPUT_OKAY); Q0[N_E]:=X
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Gruneisen parameter at infinite pressure parameter q0, or m *)
    GET_PARAMETER_VALUE(J,X,INPUT_OKAY); GAMMA_INF[N_E]:=X
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* 1st anharmonicity parameter a0 in a=a0*(V/V0)^z0 *)
    GET_PARAMETER_VALUE(J,X,INPUT_OKAY); ANH_A0[N_E]:=X
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* 2nd anharmonicity parameter z0 in a=a0*(V/V0)^z0 *)
    GET_PARAMETER_VALUE(J,X,INPUT_OKAY); ANH_Z0[N_E]:=X
  END
END;

PROCEDURE GET_ELECTRONIC_ABC(N_E:INTEGER;VAR INPUT_OKAY:BOOLEAN);
(* Gets electronic parameters A_EL, B_EL, C_EL for extended electronic
   description. That model can be used to simulate electron-phonon coupling
   behavior, such as done by Jacobs et al (2013), Phys Chem Minerals,
   40:207-227 for Platinum. a1, a2, a3, c1, c2 are per mole formula unit
 *)
VAR X:EXTENDED;
    J:INTEGER;
BEGIN
  (* Number, N_E *)
  J:=1; GET_PARAMETER_VALUE(J,X,INPUT_OKAY);
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* A_EL[N_E] parameter *)
    GET_PARAMETER_VALUE(J,X,INPUT_OKAY); A_EL[N_E]:=X
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* B_EL[N_E] parameter *)
    GET_PARAMETER_VALUE(J,X,INPUT_OKAY); B_EL[N_E]:=X
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* C_EL[N_E] parameter *)
    GET_PARAMETER_VALUE(J,X,INPUT_OKAY); C_EL[N_E]:=X
  END
END;

PROCEDURE GET_CRYSTAL_FIELD(N_E:INTEGER;VAR INPUT_OKAY:BOOLEAN);
(* Gets electronic parameters E_CF, G_CF, GAMMA_CF for electronic crystal field
   description: see me-freepascal-15jul2016.doc. The energy/degeneracy/gruneisen
   are put in array [1..m1_levels+m2_levels]
 *)
VAR X:EXTENDED;
    J:INTEGER;
BEGIN
  (* Energy of level can be M1 and M2 site levels *)
  J:=1; GET_PARAMETER_VALUE(J,X,INPUT_OKAY); E_CF[N_E]:=X;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* G_CF[N_E] degeneracy parameter *)
    GET_PARAMETER_VALUE(J,X,INPUT_OKAY); G_CF[N_E]:=X
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* GAMMA_CF[N_E] gruneisen parameter *)
    GET_PARAMETER_VALUE(J,X,INPUT_OKAY); GAMMA_CF[N_E]:=X
  END
END;

PROCEDURE GET_MODEL_PARAMETERS(VAR INPUT_OKAY,FRACTIONS_OKAY:BOOLEAN;
                               VAR NUM_LINE:INTEGER);
(* Reads model parameters *)
VAR STRIN:ANSISTRING;
    DELTA,SUM_FRACTIONS:EXTENDED;
    I,I_END,CODE,M1M2_LEVELS,N_E:INTEGER;
BEGIN
  (* We stop when input is wrong *)
  INPUT_OKAY:=TRUE; FRACTIONS_OKAY:=TRUE; NUM_LINE:=0; DELTA:=1E-08;
  (* Name of substance *)
  READLINE(LINE,NUM_LINE);
  IF LENGTHLINE>0 THEN GET_STRING(SUBSTANCE_NAME) ELSE INPUT_OKAY:=FALSE;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Molar mass *)
    READLINE(LINE,NUM_LINE);
    STRIN:=''; GET_STRING(STRIN); VAL(STRIN,MOLAR_MASS,CODE);
    IF CODE<>0 THEN INPUT_OKAY:=FALSE;
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Number of atoms in molecular formula unit *)
     READLINE(LINE,NUM_LINE);
     STRIN:=''; GET_STRING(STRIN); VAL(STRIN,N_MOL,CODE);
     IF CODE<>0 THEN INPUT_OKAY:=FALSE
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Reference energy *)
     READLINE(LINE,NUM_LINE);
     STRIN:=''; GET_STRING(STRIN); VAL(STRIN,U_REF,CODE);
     IF CODE<>0 THEN INPUT_OKAY:=FALSE
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Volume at 0 Pa, 0 K *)
     READLINE(LINE,NUM_LINE);
     STRIN:=''; GET_STRING(STRIN); VAL(STRIN,V0,CODE);
     IF CODE<>0 THEN INPUT_OKAY:=FALSE
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Static volume at 0 Pa *)
     READLINE(LINE,NUM_LINE);
     STRIN:=''; GET_STRING(STRIN); VAL(STRIN,V0_ST,CODE);
     IF CODE<>0 THEN INPUT_OKAY:=FALSE
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Equation of state type: 1=Vinet, 2=Birch-Murnaghan, 3=Keane, 4=Qin *)
     READLINE(LINE,NUM_LINE);
     STRIN:=''; GET_STRING(STRIN); VAL(STRIN,EOS_TYPE,CODE);
     IF CODE<>0 THEN INPUT_OKAY:=FALSE
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Order of Equation of state *)
     READLINE(LINE,NUM_LINE);
     STRIN:=''; GET_STRING(STRIN); VAL(STRIN,EOS_ORDER,CODE);
     IF CODE<>0 THEN INPUT_OKAY:=FALSE;
     IF EOS_ORDER>MAX_EOS_ORDER THEN INPUT_OKAY:=FALSE;
     (* Order EoS Vinet should be 3, that of Keane should be 4 *)
     CASE EOS_TYPE OF
       1:IF EOS_ORDER<>3 THEN INPUT_OKAY:=FALSE;
       3:IF EOS_ORDER<>4 THEN INPUT_OKAY:=FALSE
     END
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Bulk modulus of the static lattice *)
     READLINE(LINE,NUM_LINE);
     STRIN:=''; GET_STRING(STRIN); VAL(STRIN,K0_ST,CODE);
     IF CODE<>0 THEN INPUT_OKAY:=FALSE
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Derivative coefficients Bulk modulus of the static lattice *)
     FOR I:=1 TO EOS_ORDER-2 DO
     BEGIN
       IF INPUT_OKAY=TRUE THEN
       BEGIN
         READLINE(LINE,NUM_LINE);
         STRIN:=''; GET_STRING(STRIN); VAL(STRIN,DK0_ST[I],CODE);
         IF CODE<>0 THEN INPUT_OKAY:=FALSE
       END
     END;
     (* Keane EoS: pressure derivative of bulk modulus at infinite pressure
        should be larger than zero
      *)
     IF (EOS_TYPE=3) AND (DK0_ST[2]<1E-16) THEN INPUT_OKAY:=FALSE
  END;
  IF EOS_TYPE=2 THEN
  BEGIN
    (* Only for Birch-Murnaghan, EOS_TYPE=2, we can treat shear modulus *)
    IF INPUT_OKAY=TRUE THEN
    BEGIN
      (* Shear modulus of the static lattice *)
       READLINE(LINE,NUM_LINE);
       STRIN:=''; GET_STRING(STRIN); VAL(STRIN,G0_ST,CODE);
       IF CODE<>0 THEN INPUT_OKAY:=FALSE
    END;
    IF INPUT_OKAY=TRUE THEN
    BEGIN
      (* Derivative coefficients shear modulus of the static lattice *)
       FOR I:=1 TO EOS_ORDER-2 DO
       BEGIN
         IF INPUT_OKAY=TRUE THEN
         BEGIN
           READLINE(LINE,NUM_LINE);
           STRIN:=''; GET_STRING(STRIN); VAL(STRIN,DG0_ST[I],CODE);
           IF CODE<>0 THEN INPUT_OKAY:=FALSE
         END
       END
    END;
    IF INPUT_OKAY=TRUE THEN
    BEGIN
      (* Vibrational parameter Shear modulus, 0 K, 0 Pa *)
       READLINE(LINE,NUM_LINE);
       STRIN:=''; GET_STRING(STRIN); VAL(STRIN,NS0,CODE);
       IF CODE<>0 THEN INPUT_OKAY:=FALSE
    END;
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Landau parameters if present *)
    READLINE(LINE,NUM_LINE);
    STRIN:=''; GET_STRING(STRIN); VAL(STRIN,LANDAU_YES,CODE);
    IF CODE<>0 THEN INPUT_OKAY:=FALSE;
    IF INPUT_OKAY=TRUE THEN
    BEGIN
      DELTA:=1E-8;
      IF LANDAU_YES>DELTA THEN
      BEGIN
        I_END:=4;
        (* For 2nd order and tricritical read: a,CS,Tc0,3rd-law.
           For 1st order read: a,CS,Tr0,Q0,3rd-law
         *)
        IF ABS(LANDAU_YES-3)<DELTA THEN I_END:=5;
        FOR I:=1 TO I_END DO
        BEGIN
          IF INPUT_OKAY=TRUE THEN
          BEGIN
            READLINE(LINE,NUM_LINE);
            STRIN:=''; GET_STRING(STRIN); VAL(STRIN,LANDAU[I],CODE);
            IF CODE<>0 THEN INPUT_OKAY:=FALSE
          END
        END
      END
    END
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Magnetic description 1=YES or 0=NO *)
    READLINE(LINE,NUM_LINE);
    STRIN:=''; GET_STRING(STRIN); VAL(STRIN,MAGNETIC_YES,CODE);
    IF CODE<>0 THEN INPUT_OKAY:=FALSE;
    IF (MAGNETIC_YES>DELTA) AND (INPUT_OKAY=TRUE) THEN
    BEGIN
      (* Read magnetic constants: number of terms in McLaurin series. In Calphad
         only 3 terms are taken leading to bad convergence of the series and therefore
         a bad representation of Inden's (1981) original formalism.
       *)
      READLINE(LINE,NUM_LINE);
      STRIN:=''; GET_STRING(STRIN); VAL(STRIN,NUM_MCLAURIN_TERMS,CODE);
      IF CODE<>0 THEN INPUT_OKAY:=FALSE;
      IF NUM_MCLAURIN_TERMS<3 THEN INPUT_OKAY:=FALSE;
      IF INPUT_OKAY=TRUE THEN
      BEGIN
        (* Ratio of magnetic energy below tau=1 and the total magnetic energy *)
        READLINE(LINE,NUM_LINE);
        STRIN:=''; GET_STRING(STRIN); VAL(STRIN,INDEN_P,CODE);
        IF CODE<>0 THEN INPUT_OKAY:=FALSE     
      END;
      IF INPUT_OKAY=TRUE THEN
      BEGIN
        (* Curie/Neel temperature in Kelvin *)
        READLINE(LINE,NUM_LINE);
        STRIN:=''; GET_STRING(STRIN); VAL(STRIN,T_CURIE,CODE);
        IF CODE<>0 THEN INPUT_OKAY:=FALSE
      END;
      IF INPUT_OKAY=TRUE THEN
      BEGIN
        (* Magnetic moment in Bohr magneton *)
        READLINE(LINE,NUM_LINE);
        STRIN:=''; GET_STRING(STRIN); VAL(STRIN,MAG_MOMENT,CODE);
        IF CODE<>0 THEN INPUT_OKAY:=FALSE
      END;
      IF INPUT_OKAY=TRUE THEN
      BEGIN
        (* Exponent in the Inden expression for Cv for tau<=1 *)
        READLINE(LINE,NUM_LINE);
        STRIN:=''; GET_STRING(STRIN); VAL(STRIN,INDEN_M,CODE);
        IF CODE<>0 THEN INPUT_OKAY:=FALSE
      END;
      IF INPUT_OKAY=TRUE THEN
      BEGIN
        (* Exponent in the Inden expression for Cv for tau>1 *)
        READLINE(LINE,NUM_LINE);
        STRIN:=''; GET_STRING(STRIN); VAL(STRIN,INDEN_N,CODE);
        IF CODE<>0 THEN INPUT_OKAY:=FALSE
      END;
      IF INPUT_OKAY=TRUE THEN
      BEGIN
        (* Number of magnetic atoms per molecular formula unit *)
        READLINE(LINE,NUM_LINE);
        STRIN:=''; GET_STRING(STRIN); VAL(STRIN,NUM_MAG_ATOMS,CODE);
        IF CODE<>0 THEN INPUT_OKAY:=FALSE
      END
    END
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Electronic description *)
    READLINE(LINE,NUM_LINE);
    STRIN:=''; GET_STRING(STRIN); VAL(STRIN,ELECTRONIC_YES,CODE);
    IF CODE<>0 THEN INPUT_OKAY:=FALSE;
    IF (ELECTRONIC_YES>DELTA) AND (INPUT_OKAY=TRUE) THEN
    BEGIN
      (* Read electronic terms *)
      IF ELECTRONIC_YES=1 THEN
      BEGIN
        (* Read electronic Gruneisen parameter, free electron gas model *)
        READLINE(LINE,NUM_LINE);
        STRIN:=''; GET_STRING(STRIN); VAL(STRIN,GAMMA_EL,CODE);
        IF CODE<>0 THEN INPUT_OKAY:=FALSE;
        IF INPUT_OKAY=TRUE THEN
        BEGIN
          (* Read electronic coeffcient, free electron gas model: per mol
             formula unit
           *)
          READLINE(LINE,NUM_LINE);
          STRIN:=''; GET_STRING(STRIN); VAL(STRIN,C_EL[1],CODE);
          IF CODE<>0 THEN INPUT_OKAY:=FALSE
        END
      END
      ELSE
      BEGIN
        IF ELECTRONIC_YES=2 THEN
        BEGIN
          (* Extended free electron gas model, for electron-phonon *)
          READLINE(LINE,NUM_LINE);
          STRIN:=''; GET_STRING(STRIN); VAL(STRIN,GAMMA_EL,CODE);
          IF CODE<>0 THEN INPUT_OKAY:=FALSE;
          IF INPUT_OKAY=TRUE THEN
          BEGIN
            N_E:=0;
            REPEAT
              READLINE(LINE,NUM_LINE); N_E:=N_E+1;
              GET_ELECTRONIC_ABC(N_E,INPUT_OKAY)
            UNTIL
              (N_E>=3) OR (INPUT_OKAY=FALSE) OR (EOF(F_INPUT))
          END
        END
        ELSE
        BEGIN
          IF ELECTRONIC_YES=3 THEN
          BEGIN
            (* Crystal-Field parameters; # sites M1,M2 or M3 *)
            READLINE(LINE,NUM_LINE);
            STRIN:=''; GET_STRING(STRIN); VAL(STRIN,NUM_CF_SITES,CODE);
            IF CODE<>0 THEN INPUT_OKAY:=FALSE;
            IF (NUM_CF_SITES>DELTA) AND (INPUT_OKAY=TRUE) THEN
            BEGIN
              (* Read Crystal-Field parameters *)
              IF INPUT_OKAY=TRUE THEN
              BEGIN
                (* Crystal-Field parameters; # energy levels M1 site *)
                READLINE(LINE,NUM_LINE);
                STRIN:=''; GET_STRING(STRIN); VAL(STRIN,M1_LEVELS,CODE);
                IF CODE<>0 THEN INPUT_OKAY:=FALSE;
              END;
              IF (INPUT_OKAY=TRUE) AND (NUM_CF_SITES>1) THEN
              BEGIN
                (* Crystal-Field parameters; # energy levels M2 site *)
                READLINE(LINE,NUM_LINE);
                STRIN:=''; GET_STRING(STRIN); VAL(STRIN,M2_LEVELS,CODE);
                IF CODE<>0 THEN INPUT_OKAY:=FALSE
              END;
              IF (INPUT_OKAY=TRUE) AND (NUM_CF_SITES>2) THEN
              BEGIN
                (* Crystal-Field parameters; # energy levels M2 site *)
                READLINE(LINE,NUM_LINE);
                STRIN:=''; GET_STRING(STRIN); VAL(STRIN,M3_LEVELS,CODE);
                IF CODE<>0 THEN INPUT_OKAY:=FALSE
              END;
              IF INPUT_OKAY=TRUE THEN
              BEGIN
                (* Crystal-Field parameters;
                   # atoms per formula unit having crystal-field splitting
                 *)
                READLINE(LINE,NUM_LINE);
                STRIN:=''; GET_STRING(STRIN); VAL(STRIN,NUM_CF_ATOMS,CODE);
                IF CODE<>0 THEN INPUT_OKAY:=FALSE
              END;
              IF INPUT_OKAY=TRUE THEN
              BEGIN
                (* Magnetic correction factor for crystal-field contribution *)
                READLINE(LINE,NUM_LINE);
                STRIN:=''; GET_STRING(STRIN); VAL(STRIN,MAGNETIC_CORR_FACTOR,CODE);
                IF CODE<>0 THEN INPUT_OKAY:=FALSE
              END;
              IF INPUT_OKAY=TRUE THEN
              BEGIN
                (* Crystal-Field parameters; energy, degeneracy, -Gruneisen *)
                N_E:=0; M1M2_LEVELS:=M1_LEVELS+M2_LEVELS+M3_LEVELS;
                REPEAT
                  READLINE(LINE,NUM_LINE); N_E:=N_E+1;
                  GET_CRYSTAL_FIELD(N_E,INPUT_OKAY)
                UNTIL
                  (N_E>=M1M2_LEVELS) OR (EOF(F_INPUT)) OR (INPUT_OKAY=FALSE);
              END;
              IF INPUT_OKAY=TRUE THEN
              BEGIN
                (* Read additional free-electron gas contribution: Gruneisen parameter *)
                READLINE(LINE,NUM_LINE);
                STRIN:=''; GET_STRING(STRIN); VAL(STRIN,GAMMA_EL,CODE);
                IF CODE<>0 THEN INPUT_OKAY:=FALSE;
                IF INPUT_OKAY=TRUE THEN
                BEGIN
                  (* Read electronic coeffcient, free electron gas model *)
                  READLINE(LINE,NUM_LINE);
                  STRIN:=''; GET_STRING(STRIN); VAL(STRIN,C_EL[1],CODE);
                  IF CODE<>0 THEN INPUT_OKAY:=FALSE
                END
              END
            END
          END
          ELSE INPUT_OKAY:=FALSE
        END
      END
    END
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Cation disorder contribution *)
    READLINE(LINE,NUM_LINE);
    STRIN:=''; GET_STRING(STRIN); VAL(STRIN,CATION_DISORDER_YES,CODE);
    IF CODE<>0 THEN INPUT_OKAY:=FALSE;
    IF (CATION_DISORDER_YES>DELTA) AND (INPUT_OKAY=TRUE) THEN
    BEGIN
      (* Read parameters for cation disorder contribution:
         Interchange enthalpy and interchange entropy are expressed in the
           sitefraction (y) of the metallic ion as:
         H_int:=(h1+h2*y)*y; S_int=(s1+s2*y)*y
         CATION_DO[1]: 1st coefficient of interchange enthalpy (h1)
         CATION_DO[2]: 2nd coefficient of interchange enthalpy (h2)
         CATION_DO[3]: 1st coefficient of interchange entropy  (s1)
         CATION_DO[4]: 2nd coefficient of interchange entropy  (s2)
         CATION_DO[5]: Number of moles of tetrahedral sites per molecular formula unit
         CATION_DO[6]: Number of moles of octahedral sites per molecular formula unit
       *)
      FOR I:=1 TO 6 DO
      BEGIN
        IF INPUT_OKAY=TRUE THEN
        BEGIN
          READLINE(LINE,NUM_LINE);
          STRIN:=''; GET_STRING(STRIN); VAL(STRIN,CATION_DO[I],CODE);
          IF CODE<>0 THEN INPUT_OKAY:=FALSE
        END
      END
    END
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Monovacancy defects contribution *)
    READLINE(LINE,NUM_LINE);
    STRIN:=''; GET_STRING(STRIN); VAL(STRIN,DEFECT_YES,CODE);
    IF CODE<>0 THEN INPUT_OKAY:=FALSE;
    IF (DEFECT_YES>DELTA) AND (INPUT_OKAY=TRUE) THEN
    BEGIN
      (* Read parameters for monovacancy defects contribution:
         Helmholtz energy contribution: A=-(3/2)nRT*exp(s*eta^f-h*eta^g/T)
         DEFECT[1]: h: enthalpy, unit Kelvin
         DEFECT[2]: s: entropy,  no unit
         DEFECT[3]: f: no unit
         DEFECT[4]: g: no unit
         eta=V/V0, n=# atoms per formula unit
       *)
      FOR I:=1 TO 4 DO
      BEGIN
        IF INPUT_OKAY=TRUE THEN
        BEGIN
          READLINE(LINE,NUM_LINE);
          STRIN:=''; GET_STRING(STRIN); VAL(STRIN,DEFECT[I],CODE);
          IF CODE<>0 THEN INPUT_OKAY:=FALSE
        END
      END
    END
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Number of Einstein frequencies *)
     READLINE(LINE,NUM_LINE);
     STRIN:=''; GET_STRING(STRIN); VAL(STRIN,N_EINSTEIN,CODE);
     IF CODE<>0 THEN INPUT_OKAY:=FALSE;
     IF N_EINSTEIN>NUMBER_OF_FREQUENCIES THEN INPUT_OKAY:=FALSE
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Frequency or Gruneisen expression*)
     READLINE(LINE,NUM_LINE);
     STRIN:=''; GET_STRING(STRIN); VAL(STRIN,GRUNEISEN_TYPE,CODE);
     IF CODE<>0 THEN INPUT_OKAY:=FALSE;
     (* Only value of 1 or 2 is allowed: 1=aL'tshuler, 2=Stixrude's expression *)
     IF ABS(GRUNEISEN_TYPE-1)>DELTA THEN
       IF ABS(GRUNEISEN_TYPE-2)>DELTA THEN INPUT_OKAY:=FALSE
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Frequencies, fractions, gamma, mode-q, gamma_inf, a0, z *)
    N_E:=0;
    REPEAT
      READLINE(LINE,NUM_LINE); N_E:=N_E+1;
      GET_VIBRATIONAL(N_E,INPUT_OKAY)
    UNTIL
      (N_E>=N_EINSTEIN) OR (EOF(F_INPUT)) OR (INPUT_OKAY=FALSE);
    (* Check sum of fractions *)
    SUM_FRACTIONS:=0; DELTA:=1E-8;
    FOR I:=1 TO N_EINSTEIN DO
    BEGIN
      SUM_FRACTIONS:=SUM_FRACTIONS+FRAC[I]
    END;
    IF ABS(SUM_FRACTIONS-1)>DELTA THEN
    BEGIN
      FRACTIONS_OKAY:=FALSE; INPUT_OKAY:=FALSE
    END
  END
END;

PROCEDURE CHECK_PATH(NAME:ANSISTRING;VAR INPUT_OKAY:BOOLEAN);
(* Checks if path of the file with name NAME exists. If that is the case
   set INPUT_OKAY=TRUE otherwise set it FALSE.
 *)
VAR FIL:TEXT;
    NAME_DUMMY,PATH:ANSISTRING;
BEGIN
  (* Extract the path of the file *)
  PATH:=EXTRACTFILEPATH(NAME);
  (* Make a test name different from name because the rewrite command below will
     otherwise overwrite the existing file and empty it
   *)
  NAME_DUMMY:=PATH+'mj241019.59';
  ASSIGN(FIL,NAME_DUMMY);
  {$I-} REWRITE(FIL); {$I+}
  IF IORESULT=0 THEN
  BEGIN
    INPUT_OKAY:=TRUE; CLOSE(FIL); ERASE(FIL)
  END
  ELSE INPUT_OKAY:=FALSE
END;

PROCEDURE GET_PT_RANGES(VAR INPUT_OKAY,PATH_OKAY:BOOLEAN;VAR NUM_LINE:INTEGER);
(* Enter either:   1        isobaric calculation
                   T_start, T_end, T_step
                   P_start, P_end, p_step
                   1        1=Output to screen and file, 0=Output to file only
                   Output: path and file name
   or:             2        isothermal calculation
                   T_start, T_end, T_step
                   P_start, P_end, p_step
                   1        1=Output to screen and file, 0=Output to file only
                   Output: path and file name
   or:             4        isentropic calculation
                   S_isentrope
                   P_start, P_end, p_step
                   1        1=Output to screen and file, 0=Output to file only
                   Output: path and file name
   or:             5        Hugoniot calculation
                   T_hugo_1bar
                   P_start, P_end, p_step
                   1        1=Output to screen and file, 0=Output to file only
                   Output: path and file name
 *)
VAR STRIN:ANSISTRING;
    X:EXTENDED;
    J,CODE:INTEGER;
BEGIN
  (* Line defining temperature range and step *)
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Start temperature (1,2) or constant entropy (4) or sample temperature at
       1 bar (5)
     *)
    IF NOT EOF(F_INPUT) THEN
    BEGIN
      READLINE(LINE,NUM_LINE);
      IF LENGTHLINE>0 THEN
      BEGIN
        J:=1; GET_PARAMETER_VALUE(J,X,INPUT_OKAY);
        CASE CALCULATION_TYPE OF
          1,2:T_START:=X;
            4:S_ISENTROPE:=X;
            5:T_HUGO_1BAR:=X
        END;
        IF T_START<0 THEN INPUT_OKAY:=FALSE;
        IF INPUT_OKAY=TRUE THEN
        BEGIN
          (* Only Isobaric and isothermal calculations *)
          IF CALCULATION_TYPE<3 THEN
          BEGIN
            (* End temperature on the same line *)
            GET_PARAMETER_VALUE(J,X,INPUT_OKAY); T_END:=X;
            IF T_END<0 THEN INPUT_OKAY:=FALSE;
            IF INPUT_OKAY=TRUE THEN
            BEGIN
              (* Step temperature on the same line *)
              GET_PARAMETER_VALUE(J,X,INPUT_OKAY); T_STEP:=X
            END
          END
        END
      END
    END
  END;
  (* Line defining pressure range and step *)
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    IF NOT EOF(F_INPUT) THEN
    BEGIN
      READLINE(LINE,NUM_LINE);
      IF LENGTHLINE>0 THEN
      BEGIN
        (* Start pressure *)
        J:=1; GET_PARAMETER_VALUE(J,X,INPUT_OKAY); P_START:=X;
        IF P_START<0 THEN INPUT_OKAY:=FALSE;
        IF INPUT_OKAY=TRUE THEN
        BEGIN
          (* End pressure on the same line *)
          GET_PARAMETER_VALUE(J,X,INPUT_OKAY); P_END:=X;
          IF P_END<0 THEN INPUT_OKAY:=FALSE
        END;
        IF INPUT_OKAY=TRUE THEN
        BEGIN
          (* Step pressure on the same line *)
          GET_PARAMETER_VALUE(J,X,INPUT_OKAY); P_STEP:=X
        END;
      END
    END;
  END;
  (* Line with a number to indicate if output must be additionally written
     to screen
   *)
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    IF NOT EOF(F_INPUT) THEN
    BEGIN
    (* If output must be written to screen set a value larger than 0 *)
      READLINE(LINE,NUM_LINE);
      IF LENGTHLINE>0 THEN
      BEGIN
        STRIN:=''; GET_STRING(STRIN); VAL(STRIN,TABLE_SCREEN,CODE);
        IF CODE<>0 THEN INPUT_OKAY:=FALSE
      END;
    END
    ELSE INPUT_OKAY:=FALSE
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    IF NOT EOF(F_INPUT) THEN
    BEGIN
      (* Read output file name *)
      READLINE(LINE,NUM_LINE);
      IF LENGTHLINE>0 THEN
      BEGIN
        STRIN:=''; GET_STRING(NAME_OUTPUTFILE);
        IF NAME_OUTPUTFILE='' THEN INPUT_OKAY:=FALSE
        ELSE
        BEGIN
          (* Check if path of the output file exists. *)
          PATH_OKAY:=TRUE;
          CHECK_PATH(NAME_OUTPUTFILE,PATH_OKAY);
          IF PATH_OKAY=FALSE THEN INPUT_OKAY:=FALSE
          ELSE
          BEGIN
            (* Output file must not be the same as input file *)
            GET_PATH_INPUTFILE(NAME_INPUTFILE);
            IF NAME_OUTPUTFILE=NAME_INPUTFILE THEN INPUT_OKAY:=FALSE
          END
        END
      END;
    END
    ELSE INPUT_OKAY:=FALSE
  END
END;

PROCEDURE GET_VDOS_INPUT(VAR INPUT_OKAY,PATH_OKAY,VDOS_EXISTS:BOOLEAN;VAR NUM_LINE:INTEGER);
(* Enter: 3        VDoS
          1        Desired VDoS unit: 1=cm-1, 2=THz, 3=meV
          P,T      Pressure, temperature at which the VDoS must be calculated
          file     output file for GnuPlot
          1        0= no experimental/abinitio VDoS file present; 1=present
          (file)   If so: file with experimental/abinitio VDos
          1,2,3    If so: unit of experimental/abinitio VDoS
     or : 6        Clone VDoS
          30       Desired number of Einstein frequencies in the VDoS
          P,T      Clone conditions
          File     Output mef file
 *)
VAR STRIN:ANSISTRING;
    X:EXTENDED;
    CODE,DUMMY,J:INTEGER;
BEGIN
  VDOS_EXISTS:=TRUE;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* CALCULATION_TYPE=3: Desired VDoS unit 1=cm^-1, 2=THz, 3=meV
       CALCULATION_TYPE=6: Desired number of Einstein frequencies in the VDoS
     *)
    IF NOT EOF(F_INPUT) THEN
    BEGIN
      READLINE(LINE,NUM_LINE);
      IF LENGTHLINE>0 THEN
      BEGIN
        STRIN:=''; GET_STRING(STRIN); VAL(STRIN,DUMMY,CODE);
        IF CODE<>0 THEN INPUT_OKAY:=FALSE;
        CASE CALCULATION_TYPE OF
          3:BEGIN
              VDOS_UNIT_DESIRED:=DUMMY;
              IF (VDOS_UNIT_DESIRED>3) OR (VDOS_UNIT_DESIRED<1) THEN INPUT_OKAY:=FALSE
            END;
          6:BEGIN
              NUMEIN_CLONE:=DUMMY;
              IF NUMEIN_CLONE>=N_EINSTEIN THEN INPUT_OKAY:=FALSE
            END
        END
      END
    END
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    IF NOT EOF(F_INPUT) THEN
    BEGIN
      (* CALCULATION_TYPE=3: PRESSURE, TEMPERATURE at which the VDoS must be calculated
         CALCULATION_TYPE=6: PRESSURE, TEMPERATURE clone conditions
       *)
      READLINE(LINE,NUM_LINE);
      IF LENGTHLINE>0 THEN
      BEGIN
        (* Pressure *)
        J:=1; GET_PARAMETER_VALUE(J,X,INPUT_OKAY); PRESSURE:=X;
        IF PRESSURE<0 THEN INPUT_OKAY:=FALSE;
        IF INPUT_OKAY=TRUE THEN
        BEGIN
          (* Temperature on the same line *)
          GET_PARAMETER_VALUE(J,X,INPUT_OKAY); TEMPERATURE:=X;
          IF TEMPERATURE<0 THEN INPUT_OKAY:=FALSE
        END;
        IF CALCULATION_TYPE=6 THEN
        BEGIN
          P_CLONE:=PRESSURE; T_CLONE:=TEMPERATURE
        END
      END
    END
  END;
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* CALCULATION_TYPE=3: GnuPlot file name
       CALCULATION_TYPE=6: Output file name for the clone: a 'mef' file
     *)
    READLINE(LINE,NUM_LINE);
    IF LENGTHLINE>0 THEN
    BEGIN
      STRIN:='';
      GET_STRING(STRIN);
      IF STRIN='' THEN INPUT_OKAY:=FALSE
      ELSE
      BEGIN
        (* Check if path of the output file exists. *)
        PATH_OKAY:=TRUE;
        CHECK_PATH(STRIN,PATH_OKAY);
        IF PATH_OKAY=FALSE THEN INPUT_OKAY:=FALSE
      END;
      CASE CALCULATION_TYPE OF
        3:NAME_GNUPLOTFILE:=STRIN;
        6:BEGIN
            NAME_OUTPUTFILE:=STRIN;
             GET_PATH_INPUTFILE(NAME_INPUTFILE);
            IF NAME_OUTPUTFILE=NAME_INPUTFILE THEN INPUT_OKAY:=FALSE
          END
      END
    END
  END;
  IF CALCULATION_TYPE=3 THEN
  BEGIN
    IF INPUT_OKAY=TRUE THEN
    BEGIN
      IF NOT EOF(F_INPUT) THEN
      BEGIN
        (* Integer 0 or 1 to see if experimental/abinitio VDoS file is present *)
        READLINE(LINE,NUM_LINE);
        IF LENGTHLINE>0 THEN
        BEGIN
          STRIN:=''; GET_STRING(STRIN); VAL(STRIN,DUMMY,CODE);
          IF CODE<>0 THEN INPUT_OKAY:=FALSE;
          IF DUMMY>0 THEN VDOS_PRESENT:=TRUE
        END
      END
    END;
    IF VDOS_PRESENT=TRUE THEN
    BEGIN
      IF INPUT_OKAY=TRUE THEN
      BEGIN
        (* File Name of experimental/abinitio VDoS *)
        READLINE(LINE,NUM_LINE);
        IF LENGTHLINE>0 THEN
        BEGIN
          STRIN:=''; GET_STRING(NAME_VDOSFILE);
          IF NAME_VDOSFILE='' THEN INPUT_OKAY:=FALSE
          ELSE
          BEGIN
            (* Check if path of the output file exists. *)
            VDOS_EXISTS:=EXIST_FILE(NAME_VDOSFILE);
            IF VDOS_EXISTS=FALSE THEN INPUT_OKAY:=FALSE
          END
        END
      END;
      IF INPUT_OKAY=TRUE THEN
      BEGIN
        IF NOT EOF(F_INPUT) THEN
        BEGIN
          (* Integer 1,2,3 indicating which unit the experimental/abinitio VDoS has
             1=cm^-1, 2=THz, 3=meV
           *)
          READLINE(LINE,NUM_LINE);
          IF LENGTHLINE>0 THEN
          BEGIN
            STRIN:=''; GET_STRING(STRIN); VAL(STRIN,VDOS_UNIT_EXP,CODE);
            IF CODE<>0 THEN INPUT_OKAY:=FALSE;
            IF (VDOS_UNIT_EXP>3) OR (VDOS_UNIT_EXP<1) THEN INPUT_OKAY:=FALSE
          END
        END
      END
    END
  END
END;

PROCEDURE GET_OUTPUT_TYPE(VAR INPUT_OKAY:BOOLEAN;VAR NUM_LINE:INTEGER);
(* What kind of output: P-T table (isobar/isotherm), VDoS etc *)
VAR STRIN:ANSISTRING;
    CODE:INTEGER;
BEGIN
  (* Line defining CALCULATION_TYPE *)
  IF NOT EOF(F_INPUT) THEN
  BEGIN
    (* P-T tables: 1=Isobars, 2=isotherms, 3=VDoS, 4=isentrope, 5=hugoniot,
       6=clone
     *)
    READLINE(LINE,NUM_LINE);
    IF LENGTHLINE>0 THEN
    BEGIN
      STRIN:=''; GET_STRING(STRIN); VAL(STRIN,CALCULATION_TYPE,CODE);
      IF CODE<>0 THEN INPUT_OKAY:=FALSE;
      (* CALCULATION_TYPE is defined up to 5 *)
      IF CALCULATION_TYPE>6 THEN INPUT_OKAY:=FALSE
    END;
  END
END;

PROCEDURE WARN_WRONG_INPUT(CALCULATION_TYPE,NUM_LINE:INTEGER;FRACTIONS_OKAY,PATH_OKAY,VDOS_EXISTS:BOOLEAN);
(* Warning for wrong input *)
BEGIN
  WRITELN; WRITELN;
  WRITELN('--------------------------------------------------------------');
  WRITELN('Input file: ',NAME_INPUTFILE);
  WRITELN('Error in line number: ',NUM_LINE);
  IF EOS_ORDER>MAX_EOS_ORDER THEN
  BEGIN
    WRITELN('Equation of State order exceeds ',MAX_EOS_ORDER);
    WRITELN('Change MAX_EOS_ORDER in unit variables.pas and re-compile')
  END;
  IF (EOS_TYPE=1) AND (EOS_ORDER>3) THEN
  BEGIN
    WRITELN('Order of Equation of State by Vinet not equal 3')
  END;
  IF EOS_TYPE=3 THEN
  BEGIN
    IF EOS_ORDER>4 THEN WRITELN('Order of Equation of State by Keane not equal 4');
    IF DK0_ST[2]<1E-16 THEN
    BEGIN
      WRITELN('Keane EoS: pressure derivative of bulk modulus as infinite pressure');
      WRITELN('should be larger than zero')
    END
  END;
  IF N_EINSTEIN>NUMBER_OF_FREQUENCIES THEN
  BEGIN
    WRITELN('Number of Einstein temperatures exceeds ',NUMBER_OF_FREQUENCIES);
    WRITELN('Change NUMBER_OF_FREQUENCIES in unit variables.pas and re-compile')
  END;
  IF FRACTIONS_OKAY=FALSE THEN
  BEGIN
    WRITELN('Sum of fractions not equal to one...');
    WRITELN('Check number of Einstein temperatures...')
  END;
  IF PATH_OKAY=FALSE THEN WRITELN('Path of output file is wrong...');
  IF NAME_OUTPUTFILE=NAME_INPUTFILE THEN
  BEGIN
    WRITELN('Name of output file must differ from that of input file...')
  END;
  IF CALCULATION_TYPE=3 THEN
  BEGIN
    IF VDOS_EXISTS=FALSE THEN WRITELN('Cannot find file with VDoS points...')
  END;
  WRITELN('Program terminates...');
  WRITELN('--------------------------------------------------------------');
  WRITELN; WRITELN
END;

PROCEDURE READ_EXP_VDOS(VAR NUM_LINE:INTEGER;VAR INPUT_OKAY:BOOLEAN);
(* If present read file with experimental/abinitio VDoS and check file *)
VAR X:EXTENDED;
    J,NUM:INTEGER;
BEGIN
  SET_READ_FILE(F_INPUT,NAME_VDOSFILE);
  (* Reset line counter for reading VDoS file *)
  NUM_LINE:=0; NUM:=0;
  WHILE (NOT EOF(F_INPUT)) AND (INPUT_OKAY=TRUE) DO
  BEGIN
    (* VDoS: X_EXP, Y_EXP *)
    READLINE(LINE,NUM_LINE);
    IF LENGTHLINE>0 THEN
    BEGIN
      (* When the last line is a comment line EOF(F_INPUT)=True it has no values *)
      J:=1; DETECTNOTBLANK(LINE,J);
      IF LINE[J]<>'#' THEN
      BEGIN
        (* X_EXP *)
        J:=1; GET_PARAMETER_VALUE(J,X,INPUT_OKAY);
        NUM:=NUM+1; X_EXP[NUM]:=X;
        IF X<0 THEN INPUT_OKAY:=FALSE;
        IF INPUT_OKAY=TRUE THEN
        BEGIN
          (* Y_EXP on the same line *)
          GET_PARAMETER_VALUE(J,X,INPUT_OKAY); Y_EXP[NUM]:=X;
          IF X<0 THEN INPUT_OKAY:=FALSE
        END
      END
    END
  END;
  NUM_VDOS_POINTS:=NUM;
  CLOSE(F_INPUT)
END;

PROCEDURE WARN_WRONG_VDOS(NUM_LINE:INTEGER;VDOS_EXISTS:BOOLEAN);
(* When input of VDoS file is not correct *)
BEGIN
  WRITELN; WRITELN;
  WRITELN('--------------------------------------------------------------');
  WRITELN('VDoS file           : ',NAME_VDOSFILE);
  WRITELN('Error in line number: ',NUM_LINE);
  IF VDOS_EXISTS=FALSE THEN
  BEGIN
    WRITELN('Experimental/abinitio VDoS file cannot be found...')
  END;
  WRITELN('Program terminates...');
  WRITELN('--------------------------------------------------------------');
  WRITELN; WRITELN
END;

PROCEDURE READ_INPUT_FILE(VAR INPUT_OKAY:BOOLEAN);
VAR NUM_LINE:INTEGER;
    FRACTIONS_OKAY,PATH_OKAY,VDOS_EXISTS:BOOLEAN;
BEGIN
  (* Initialize model parameters: unit variables.pas *)
  INITIALIZE_VARIABLES;
  FRACTIONS_OKAY:=TRUE; PATH_OKAY:=TRUE; VDOS_EXISTS:=FALSE; CALCULATION_TYPE:=0;
  (* Set input file open for reading *)
  SET_READ_FILE(F_INPUT,NAME_INPUTFILE);
  (* Read model parameters from the input file *)
  NUM_LINE:=0;
  GET_MODEL_PARAMETERS(INPUT_OKAY,FRACTIONS_OKAY,NUM_LINE);
  IF INPUT_OKAY=TRUE THEN
  BEGIN
    (* Get type of output=CALCULATION_TYPE: 1,2=P-T table, 3=VDoS etc: *)
    GET_OUTPUT_TYPE(INPUT_OKAY,NUM_LINE);
    IF INPUT_OKAY=TRUE THEN
    BEGIN
      CASE CALCULATION_TYPE OF
        1,2,4,5:GET_PT_RANGES(INPUT_OKAY,PATH_OKAY,NUM_LINE);
        3,6:GET_VDOS_INPUT(INPUT_OKAY,PATH_OKAY,VDOS_EXISTS,NUM_LINE)
      END
    END;
    IF INPUT_OKAY=FALSE THEN
    BEGIN
      WARN_WRONG_INPUT(CALCULATION_TYPE,NUM_LINE,FRACTIONS_OKAY,PATH_OKAY,VDOS_EXISTS)
    END
  END
  ELSE WARN_WRONG_INPUT(CALCULATION_TYPE,NUM_LINE,FRACTIONS_OKAY,PATH_OKAY,VDOS_EXISTS);
  (* Close input file *)
  CLOSE(F_INPUT);
  IF VDOS_PRESENT=TRUE THEN
  BEGIN
    IF INPUT_OKAY=TRUE THEN
    BEGIN
      (* Input file has been read. In case an experimental/abinitio VDoS is
         defined read the values
       *)
      NUM_LINE:=0;
      READ_EXP_VDOS(NUM_LINE,INPUT_OKAY);
      IF INPUT_OKAY=FALSE THEN WARN_WRONG_VDOS(NUM_LINE,VDOS_EXISTS)
    END
  END
END;

END.
